using System;
using System.Collections.Generic;

namespace Encompass
{
    internal class EntityManager
    {
        private readonly Dictionary<Guid, Entity> IDToEntity = new Dictionary<Guid, Entity>();

        private readonly HashSet<Guid> entitiesMarkedForDestroy = new HashSet<Guid>();

        private readonly ComponentManager componentManager;

        public EntityManager(ComponentManager componentManager)
        {
            this.componentManager = componentManager;
        }

        public Entity CreateEntity()
        {
            var id = NextID();
            var entity = new Entity(id);
            IDToEntity[id] = entity;
            componentManager.RegisterEntity(id);
            return entity;
        }

        public bool EntityExists(Guid id)
        {
            return IDToEntity.ContainsKey(id);
        }

        public Entity GetEntity(Guid id)
        {
            return IDToEntity[id];
        }

        public void MarkForDestroy(Guid entityID)
        {
            entitiesMarkedForDestroy.Add(entityID);
        }

        public void DestroyMarkedEntities()
        {
            foreach (var entityID in entitiesMarkedForDestroy)
            {
                componentManager.MarkAllComponentsOnEntityForRemoval(entityID);
                IDToEntity.Remove(entityID);
                componentManager.RegisterDestroyedEntity(entityID);
            }

            entitiesMarkedForDestroy.Clear();
        }

        private Guid NextID()
        {
            return Guid.NewGuid();
        }
    }
}
